@@ -11,6 +11,7 @@ import multiprocessing as mp
 import time
 from cognet.util import embed_to_pca
 import pkgutil
+import os
 
 class cognet:
     """Aggregate related Qnet functions
@@ -73,7 +74,7 @@ class cognet:
             if all(x is None for x in [model.samples, samples_file]):
                 raise ValueError("Please input samples if loading model after loading qnet")
             elif model.samples is not None:
-                samples = pd.DataFrame(model.samples)
+                samples = pd.DataFrame(model.samples, columns=model.features)
             elif samples_file is not None:
                 samples = pd.read_csv(samples_file)
                 
@@ -434,8 +435,7 @@ class cognet:
             name_pref,
             out_dir,
             pca_model=False,
-            EMBED_BINARY=None  
-    ):
+            EMBED_BINARY=None):
         '''
         embed data
 
@@ -452,7 +452,7 @@ class cognet:
             FILE = infile
 
             if EMBED_BINARY is None:
-                EMBED = pkg.util.get_data("cognet.bin", "__embed__.so") 
+                EMBED = pkgutil.get_data("cognet.bin", "__embed__.so") 
             else:
                 EMBED = EMBED_BINARY
             DATAFILE = out_dir + 'data_' +yr
@@ -629,8 +629,7 @@ class cognet:
           return_dict (dict): dict containing results
           MISSING_VAL (float): default dissonance value
         '''
-        if all(x is not None for x in [self.samples, self.features, 
-                                    self.poles]):
+        if all(x is not None for x in [self.samples, self.features]):
             s = self.samples_as_strings[sample_index]
             if self.polar_indices is None:
                 self.polar_indices = range(len(s))
@@ -648,14 +647,11 @@ class cognet:
             if return_dict is not None:
                 return_dict[sample_index] = diss[self.polar_indices]
             return diss[self.polar_indices]
-
-        elif self.poles is None:
-            raise ValueError("set_poles first!")
         else:
             raise ValueError("load_data first!")
     
     def dissonance_matrix(self,
-                        output_file=None,
+                        output_file='/example_results/DISSONANCE_matrix.csv',
                         n_jobs=28):
         '''get the dissonance for all samples
 
@@ -663,9 +659,6 @@ class cognet:
           output_file (str): directory and/or file for output
           n_jobs (int): number of jobs for pdqm
         '''
-        if output_file is None:
-            output_file = '/example_results/DISSONANCE_matrix.csv'
-            
         manager = mp.Manager()
         return_dict = manager.dict()
         processes = []
@@ -839,3 +832,119 @@ class cognet:
             result.to_csv(out_file)
         
         return result.rederr.mean(), result.rand_err.mean()
+    
+    def dmat_filewriter(self,
+                        pyfile,
+                        QNETPATH,
+                        MPI_SETUP_FILE="mpi_setup.sh",
+                        MPI_RUN_FILE="mpi_run.sh",
+                        MPI_LAUNCHER_FILE="mpi_launcher.sh",
+                        YEARS='2016',
+                        NODES=4,
+                        T=12,
+                        num_samples=None,
+                        OUTFILE='tmp_distmatrix.csv'):
+        if all(x is not None for x in [self.poles_dict,self.features,
+                                       self.qnet, self.cols]):
+            if num_samples is not None:
+                self.set_nsamples(num_samples)
+            
+            tmp_path = "mpi_tmp/"
+            pd.DataFrame(self.samples_as_strings).to_csv(tmp_path+"tmp_samples_as_strings.csv", header=None, index=None)
+            w = self.samples.index.size
+            
+            if not os.path.exists(tmp_path):
+                os.makedirs(tmp_path)
+            with open(tmp_path+pyfile, 'w+') as f:
+                f.writelines(["from mpi4py.futures import MPIPoolExecutor\n",
+                              "import numpy as np\n",
+                              "import pandas as pd\n",
+                              "from quasinet.qnet import Qnet, qdistance, load_qnet, qdistance_matrix\n",
+                              "from quasinet.qsampling import qsample, targeted_qsample\n\n",
+                              "qnet=load_qnet(\'{}\')\n".format(QNETPATH)])
+
+                f.writelines(["w = {}\n".format(w),
+                              "h = w\n",
+                              "p_all = pd.read_csv(\"tmp_samples_as_strings.csv\")\n\n"])
+
+                f.writelines(["def distfunc(x,y):\n",
+                              "\td=qdistance(x,y,qnet,qnet)\n",
+                              "\treturn d\n\n"])
+
+                f.writelines(["def dfunc_line(k):\n",
+                              "\tline = np.zeros(w)\n",
+                              "\ty = np.array(p_all.iloc[k])\n",
+                              "\tfor j in range(w):\n",
+                              "\t\tif j > k:\n",
+                              "\t\t\tx = np.array(p_all.iloc[j])\n",
+                              "\t\t\tline[j] = distfunc(x, y)\n",
+                              "\treturn line\n\n"])
+
+                f.writelines(["if __name__ == '__main__':\n",
+                              "\twith MPIPoolExecutor() as executor:\n",
+                              "\t\tresult = executor.map(dfunc_line, range(h))\n",
+                              "\t\tpd.DataFrame(result).to_csv(\'{}\',index=None,header=None)".format(OUTFILE)])
+
+            # with open(MPI_LAUNCHER_FILE, 'wx') as ml:
+            #     ml.writelines(["#!/bin/bash\n\n",
+            #                    "PROG=\"\"\n",
+            #                    "RSTR=`cat /dev/urandom | tr -dc \'a-zA-Z0-9\' | fold -w 20  | head -n 1 | cut -c 1-6`\n",
+            #                    "JOBN=IXC\"$RSTR\"\n",
+            #                    "TIME=10\n",
+            #                    "NCOR=28\n",
+            #                    "MEMR=10\n",
+            #                    "NODE=1\n",
+            #                    "EXECUTE=0\n",
+            #                    "DEPEND=""\n",
+            #                    "CDIR=`pwd`\n",
+            #                    "DRY_RUN=0\n",
+            #                    "ANYDEPEND=""\n",
+            #                    "PART='sandyb'\n"
+            #                    ])
+                
+            with open(tmp_path+MPI_SETUP_FILE, 'w+') as ms:
+                ms.writelines(["#!/bin/bash\n",
+                               "YEAR=$1\n\n",
+                               "if [ $# -gt 1 ] ; then\n",
+                               "\tNODES=$2\n",
+                               "else\n",
+                               "\tNODES=3\n",
+                               "fi\n",
+                               "if [ $# -gt 2 ] ; then\n",
+                               "\tNUM=$3\n",
+                               "else\n",
+                               "\tNUM='all'\n",
+                               "fi\n",
+                               "if [ $# -gt 3 ] ; then\n",
+                               "\tPROG=$4\n",
+                               "else\n",
+                               "\tPROG=$(tty)\n",
+                               "fi\n\n",
+                               "NUMPROC=`expr 28 \* $NODES`\n",
+                               "echo \"module load midway2\" >> $PROG\n",
+                               "echo \"module unload python\" >> $PROG\n",
+                               "echo \"module unload openmpi\" >> $PROG\n",
+                               "echo \"module load python/anaconda-2020.02\" >> $PROG\n",
+                               "echo \"module load mpi4py\" >> $PROG\n",
+                               "echo \"date; mpiexec -n \"$NUMPROC\" python3 -m mpi4py.futures {}; date\"  >> $PROG\n".format(pyfile),
+                                ])
+
+            with open(tmp_path+MPI_RUN_FILE, 'w+') as mr:
+                mr.writelines(["#!/bin/bash\n",
+                               "YEARS=\'{}\'\n".format(YEARS),
+                               "# nodes requested\n",
+                               "NODES={}\n".format(NODES),
+                               "# time requested\n",
+                               "T={}\n".format(T),
+                               "NUM=\'all\'\n",
+                               "LAUNCH=\'../mpi_launcher.sh\'\n\n",
+                               "for yr in `echo $YEARS`\n",
+                               "do\n",
+                               "\techo $yr\n",
+                               "\t./{} $yr $NODES $NUM tmp_\"$yr\"\n".format(MPI_SETUP_FILE),
+                               "\t$LAUNCH -P tmp_\"$yr\" -F -T $T -N \"$NODES\" -C 28 -p broadwl -J ACRDALL_\"$yr\" -M 56\n",
+                               "done\n",
+                               "rm tmp*\n"])
+        else:
+            raise ValueError("load data first!")
+        print("running")
